@@ -1,7 +1,7 @@
 #include <GToolKit/GToolKit.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSArray.h>
-#include <Foundation/NSObjCRuntime.h>
+#include <Foundation/NSUtilities.h>
 
 int main (int argc, char *argv[])
 {
