@@ -641,7 +641,7 @@
     return [self initWithGtk:gtk_color_selection_dialog_new(NSString_to_String( title))];
 }
 #define gtk	((GtkColorSelectionDialog *) gtk)
-- (GTKWidget *) colorsel
+- (GTKColorSelection *) colorSelection
 {
     return Gtk_to_Object(gtk->colorsel, 0);
 }
@@ -1828,11 +1828,9 @@
 {
     gtk_menu_factory_add_subfactory(gtk,[ subfactory  gtk],NSString_to_String( path));
 }
-- (void) removePaths:(NSString **) paths npaths:(int) npaths
+- (void) removePaths:(NSArray  *) paths npaths:(int) npaths
 {
-    char *_outval_1;
-    gtk_menu_factory_remove_paths(gtk,&_outval_1, npaths);
-    * paths  = String_to_NSString(_outval_1);
+    gtk_menu_factory_remove_paths(gtk,(char **)NSArray_to_StrVec( paths ), npaths);
 }
 - (void) removeEntries:(GtkMenuEntry *) entries nentries:(int) nentries
 {
@@ -1846,6 +1844,50 @@
 {
     return gtk_menu_factory_find(gtk,NSString_to_String( path));
 }
+#include <stdio.h>
+#include <Foundation/NSAutoreleasePool.h>
+#define gtk	((GtkMenuFactory *) gtk)
+ 
+static void menu_forward_signal (GtkWidget *widget, gpointer user_data)
+{
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+    id target = gtk_object_get_data((GtkObject *) widget, "_menu_target");
+ 
+#ifdef DEBUG
+    fprintf(stderr, "(menu_forward_signal init) gtk = %p, action [%p %s]\n",
+	    widget, target, [NSStringFromSelector(user_data) cString]);
+#endif
+    [target performSelector:user_data withObject:Gtk_to_Object(widget, NULL)];
+    [pool release];
+#ifdef DEBUG
+    fprintf(stderr, "(menu_forward_signal exit) gtk = %p\n", widget);
+#endif
+}
+- (void) addEntries:(GtkMenuEntry *) entries forTarget:target
+{
+    GtkMenuEntry *base = entries;
+    int index, count = 0;
+ 
+    while (entries->path) ++entries, ++count;
+    for (index = 0; index != count; ++index)
+	base[index].callback = menu_forward_signal;
+ 
+    gtk_menu_factory_add_entries(gtk, base, count);
+ 
+    for (index = 0; index != count; ++index)
+	if (base[index].widget)
+	    gtk_object_set_data((GtkObject *) base[index].widget,
+				"_menu_target", target);
+}
+- (GTKAcceleratorTable *) acceleratorTable
+{
+    return Gtk_to_Object(gtk->table, "GTKAcceleratorTable");
+}
+- (GTKWidget *) widget
+{
+    return Gtk_to_Object(gtk->widget, 0);
+}
+#undef gtk
 @end
 #include <GToolKit/GToolKit.h>
 
@@ -3794,10 +3836,36 @@ static void destroy_notify (GTKObject *self)
     return gtk_window_activate_default(gtk);
 }
 #define gtk	((GtkWindow *) gtk)
+static gint focus_in_window (GtkWidget *win, GdkEvent *event, GTKWindow *self)
+{
+    [[NSNotificationCenter defaultCenter] postNotificationName:
+	GTKWindowDidBecomeMainNotification object:self];
+    return NO;
+}
+static gint focus_out_window (GtkWidget *win, GdkEvent *event, GTKWindow *self)
+{
+    [[NSNotificationCenter defaultCenter] postNotificationName:
+	GTKWindowDidResignMainNotification object:self];
+    return NO;
+}
+static void destroy_window (GtkWidget *win, GTKWindow *self)
+{
+    [[NSNotificationCenter defaultCenter] postNotificationName:
+	GTKWindowWillCloseNotification object:self];
+}
  - initWithGtk:(gpointer) _gtk
 {
     [super initWithGtk:_gtk];
     [self ref];		// first reference is missing (due to floating flag)
+ 
+    gtk_signal_connect(_gtk, "focus_in_event",
+			(GtkSignalFunc) focus_in_window, self);
+    gtk_signal_connect(_gtk, "focus_out_event",
+			(GtkSignalFunc) focus_out_window, self);
+    gtk_signal_connect(_gtk, "destroy", (GtkSignalFunc) destroy_window, self);
+    [[NSNotificationCenter defaultCenter] postNotificationName:
+	GTKWindowWillOpenNotification object:self];
+ 
     return self;
 }
 - (NSString *) title
@@ -3813,4 +3881,9 @@ static void destroy_notify (GTKObject *self)
     return Gtk_to_Object(gtk->default_widget, 0);
 }
 #undef gtk
+
+NSString *GTKWindowDidBecomeMainNotification = @"GTKWindowDidBecomeMain";
+NSString *GTKWindowDidResignMainNotification = @"GTKWindowDidResignMain";
+NSString *GTKWindowWillCloseNotification = @"GTKWindowWillClose";
+NSString *GTKWindowWillOpenNotification = @"GTKWindowWillOpen";
 @end
